/* TODO: This should move to a xs.app.setup, so it is easier for us to not ship this. */
import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import groovy.rainmaker.constants.*;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.io.IOException;



def taskLogger;
def lang;

def setupClasspath() {
    def f = new File(config.dependencies.'ibm:xs.app'[] + "/.zero/private/resolved.properties")
    
    def lib = new File(config.dependencies.'ibm:xs.app'[] + "/../lib/ogclient.jar")
    if ( !lib.exists() ) {//WAS installation
       lib = new File(config.dependencies.'ibm:xs.app'[] + "/lib/ogclient.jar")
    }
    
    def fs = new FileInputStream(f.absolutePath)
    def p = new Properties()

    p.load(fs)

    if (!p["classpath.cli"].contains(lib.absolutePath)) {
        p["classpath.cli"] += System.getProperties().getProperty("path.separator") + lib.absolutePath
        
        def os = new FileOutputStream(f.absolutePath)

        p.store(os, "")
    }
}

def broadcastInfo(message) {
    Output.info(message);
    log(message);
}

def broadcastError(message) {
    Output.error(message);
    log(message);
}

def log(message) {
    taskLogger.append(getNow()+" :: "+message+"\n");
}

def getNow() {
    SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
    return formatter.format(Calendar.getInstance().getTime());
}

def onCliTask() {
    taskLoggerPath = config.dependencies.'ibm:xs.app'[] + "/logs/startConsoleServer.log"
    lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
    taskLogger = new File(taskLoggerPath);

    Socket socket = null;
    Exception lastException = null;
    try {
        /*
         * Wait a maximum of 2 minutes for the console to come up.
         */
        expireTime = System.currentTimeMillis() + (2*60*1000);
        while (System.currentTimeMillis() <= expireTime) {
            try {
                socket = new Socket();
                SocketAddress address = new InetSocketAddress('localhost', config.https.port[].toInteger());
                socket.connect(address, 1000);
                lastException = null;
                break;
            } catch (IOException e) {
                lastException = e;
                logger.FINE { "Console is not up, checking again in 5 seconds." }
            } finally {
                if (socket != null) {
                    try {
                        socket.close();
                    } catch (Exception e){}
                }
            }
            
            retryCount--;
            sleep(5000);
        }
        
        //
        // We need to save off a version of the HTTPS configuration we started with. This needs to happen
        // b/c when we change the settings in the UI, the 'https-config.config' file gets updated and when
        // stopConsoleServer.bat|sh is called, it could try to use the the new SSL/TLS settings that will
        // fail with the current running process (the new settings don't take effect until restart).
        //
        def startFile   = new File((String) config.xsa.httpsStartFile[]);
        def configFile  = new File((String) config.xsa.httpsConfigFile[]);
        def defaultFile = new File((String) config.xsa.httpsDefaultFile[]);
        if (configFile.exists()) {
            startFile << configFile.text;
        } else if(defaultFile.exists()) {
            startFile << defaultFile.text;
        }
        
        if (lastException != null) {
            /*
             * We timed out waiting to connect to the console. It may still be starting.
             */
            broadcastInfo(MessageUtils.message(lang, 'XSW0168'));
            broadcastInfo(MessageUtils.message(lang, 'XSW0161', [config.http.port[].toString()]));
            broadcastInfo(MessageUtils.message(lang, 'XSW0162', [config.https.port[].toString()]));

            throw new Exception("Timed out waiting to connect to the console.", lastException);
        } else {
            /*
             * We connected to the console successfully.
             */
            broadcastInfo(MessageUtils.message(lang, 'XSW0163'));
            broadcastInfo(MessageUtils.message(lang, 'XSW0161', [config.http.port[].toString()]));
            broadcastInfo(MessageUtils.message(lang, 'XSW0162', [config.https.port[].toString()]));
        }
            
    } catch(Exception e) {
        logger.FINE { CleanStackTrace.cleanStackTrace(e) };
    }
}
